.yui3-skin-sam .yui3-console-separate {
    position:absolute;
    right:1em;
    top:1em;
    z-index:999;
}

.yui3-skin-sam .yui3-console-inline {
    /* xbrowser inline-block styles */
    display: -moz-inline-stack; /* FF2 */
    display: inline-block;
    *display: inline; /* IE 7- (with zoom) */
    zoom: 1;
    vertical-align: top;
}
.yui3-skin-sam .yui3-console-inline .yui3-console-content {
    position: relative;
}

.yui3-skin-sam .yui3-console-content {
    background: #777;
    _background: #D8D8DA url(bg.png) repeat-x 0 0;
    font: normal 13px/1.3 Arial, sans-serif;
    text-align: left;

    border: 1px solid #777;
    border-radius: 10px;
    -moz-border-radius: 10px;
    -webkit-border-radius: 10px;
}

.yui3-skin-sam .yui3-console-hd,
.yui3-skin-sam .yui3-console-bd,
.yui3-skin-sam .yui3-console-ft {
    position: relative;
}

.yui3-skin-sam .yui3-console-hd,
.yui3-skin-sam .yui3-console-ft .yui3-console-controls {
    text-align: right;
}

.yui3-skin-sam .yui3-console-hd {
    background: #D8D8DA url(bg.png) repeat-x 0 0;
    padding: 1ex;

    border: 1px solid transparent;
    _border: 0 none;
    border-top-right-radius: 10px;
    border-top-left-radius: 10px;
    -moz-border-radius-topright: 10px;
    -moz-border-radius-topleft: 10px;
    -webkit-border-top-right-radius: 10px;
    -webkit-border-top-left-radius: 10px;
}

.yui3-skin-sam .yui3-console-bd {
    background: #fff;
    border-top: 1px solid #777;
    border-bottom: 1px solid #777;
    color: #000;
    font-size: 11px;
    overflow: auto;
    overflow-x: auto;
    overflow-y: scroll;
    _width: 100%;
}

.yui3-skin-sam .yui3-console-ft {
    background: #D8D8DA url(bg.png) repeat-x 0 0;

    border: 1px solid transparent;
    _border: 0 none;
    border-bottom-right-radius: 10px;
    border-bottom-left-radius: 10px;
    -moz-border-radius-bottomright: 10px;
    -moz-border-radius-bottomleft: 10px;
    -webkit-border-bottom-right-radius: 10px;
    -webkit-border-bottom-left-radius: 10px;
}

.yui3-skin-sam .yui3-console-controls {
    padding: 4px 1ex;
    zoom: 1;
}

.yui3-skin-sam .yui3-console-title {
    color: #000;
    display: inline;
    float: left;
    font-weight: bold;
    font-size: 13px;
    height: 24px;
    line-height: 24px;
    margin: 0;
    padding-left: 1ex;
}

.yui3-skin-sam .yui3-console-pause-label {
    float: left;
}
.yui3-skin-sam .yui3-console-button {
    line-height: 1.3;
}

.yui3-skin-sam .yui3-console-collapsed .yui3-console-bd,
.yui3-skin-sam .yui3-console-collapsed .yui3-console-ft {
    display: none;
}
.yui3-skin-sam .yui3-console-content.yui3-console-collapsed {
    -webkit-border-radius: 0;
}
.yui3-skin-sam .yui3-console-collapsed .yui3-console-hd {
    border-radius: 10px;
    -moz-border-radius: 10px;
    -webkit-border-radius: 0;
}

/* Log entries */
.yui3-skin-sam .yui3-console-entry {
    border-bottom: 1px solid #aaa;
    min-height: 32px;
    _height: 32px;
}

.yui3-skin-sam .yui3-console-entry-meta {
    margin: 0;
    overflow: hidden;
}

.yui3-skin-sam .yui3-console-entry-content {
    margin: 0;
    padding: 0 1ex;
    white-space: pre-wrap;
    word-wrap: break-word;
}

.yui3-skin-sam .yui3-console-entry-meta .yui3-console-entry-src {
    color: #000;
    font-style: italic;
    font-weight: bold;
    float: right;
    margin: 2px 5px 0 0;
}
.yui3-skin-sam .yui3-console-entry-meta .yui3-console-entry-time {
    color: #777;
    padding-left: 1ex;
}
.yui3-skin-sam .yui3-console-entry-warn .yui3-console-entry-meta .yui3-console-entry-time {
    color: #555;
}

.yui3-skin-sam .yui3-console-entry-info .yui3-console-entry-meta .yui3-console-entry-cat,
.yui3-skin-sam .yui3-console-entry-warn .yui3-console-entry-meta .yui3-console-entry-cat,
.yui3-skin-sam .yui3-console-entry-error .yui3-console-entry-meta .yui3-console-entry-cat {
    display: none;
}
.yui3-skin-sam .yui3-console-entry-warn {
    background: #aee url(warn_error.png) no-repeat -15px 15px;
}
.yui3-skin-sam .yui3-console-entry-error {
    background: #ffa url(warn_error.png) no-repeat 5px -24px;
    color: #900;
}
.yui3-skin-sam .yui3-console-entry-warn .yui3-console-entry-content,
.yui3-skin-sam .yui3-console-entry-error .yui3-console-entry-content {
    padding-left: 24px;
}
.yui3-skin-sam .yui3-console-entry-cat {
    text-transform: uppercase;
    padding: 1px 4px;
    background-color: #ccc;
}
.yui3-skin-sam .yui3-console-entry-info .yui3-console-entry-cat {
    background-color: #ac2;
}
.yui3-skin-sam .yui3-console-entry-warn .yui3-console-entry-cat {
    background-color: #e81;
}
.yui3-skin-sam .yui3-console-entry-error .yui3-console-entry-cat {
    background-color: #b00;
    color: #fff;
}

.yui3-skin-sam .yui3-console-hidden { display: none; }
